/* 
   COPYRIGHT 2004 KRUPCZAK.ORG, LLC.

   THIS SOFTWARE IS THE CONFIDENTIAL AND PROPRIETARY PRODUCT OF 
   KRUPCZAK.ORG, LLC.  ANY UNAUTHORIZED USE, REPRODUCTION, OR
   TRANSFER OF THIS SOFTWARE IS STRICTLY PROHIBITED.  (SUBJECT TO LIMITED
   DISTRIBUTION AND RESTRICTED DISCLOSURE ONLY.)  ALL RIGHTS RESERVED.
*/

/* file - testparse.c */
/* rdk@krupczak.org */

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "xmp.h"

int xmpd_verbose = 0; /* for xmp.c to link with us */
int xmp_session_verbose = 0;

/* ************************************************************* */

int traverseAttributes(xmlNodePtr ptr)
{
     struct _xmlAttr *a;

     if (ptr == NULL) {
        return -1;
     }

     if (ptr->properties == NULL) {
        printf("\t%s has no attributes\n",ptr->name);
        return -1;
     }

     a = ptr->properties;

     while (a != NULL) {

       if (a->children != NULL) {
          printf("\tAttribute:%s=%s\n",a->name,a->children->content);
       }
       else {
          printf("\tAttribute %s, no-value\n",a->name);
       }

       a = a->next;
     }

     return 1;
}

/* ************************************************************* */
/* traverse a node and children */

int traverseNode(xmlNodePtr ptr, int indent)
{
    xmlNodePtr child;
    int i;

    if (ptr == NULL)
       return 0;

#if 0
    /* cant skip text nodes as they may be the content for a parent node */
    if (ptr->type == XML_TEXT_NODE)
       return 1;
#endif

    for (i=0;i<indent; i++)
        printf(" ");
    printf("Node:'%s', Type:%d Content:'%s'\n",
           ptr->name,ptr->type,ptr->content);
    traverseAttributes(ptr);

    child = ptr->children;

    while (child != NULL) {
      traverseNode(child,indent+2);
      child = child->next;
    }

    return 1;
}


/* ************************************************************* */

int main(int argc, char *argv[])
{
     xmpPdu_t pdu;

     if (argc != 2) {
        fprintf(stderr,"Usage: %s xml-file-name\n",argv[0]);
        exit(0);
     }

     fprintf(stderr,"Size of wireheader is %d\n",sizeof(xmpWireHdr_t));

     printf("testparse: going to parse XMP message in %s\n",argv[1]);

     if (parseXmpMessageByFilename(argv[1],&pdu) < 0) {
        printf("testparse: parse %s failed\n",argv[1]);
     }
     else {
        dumpInternalXmpMessage(&pdu);
     }
     
}

/* ************************************************************* */


#if 0
     doc = xmlParseFile(argv[1]);

     if (doc == NULL) {
        fprintf(stderr,"Document not parsed correctly\n");
        exit(-1);
     }

     printf("testparse: parsed xml file %s\n",argv[0]);
     printf("testparse: doc name %s, compression:%d\n",doc->name,
            doc->compression);
     printf("testparse: xml version %s, URL:%s\n\n",doc->version,doc->URL);

     if ((cur = xmlDocGetRootElement(doc)) == NULL) {
        fprintf(stderr,"empty document\n");
        xmlFreeDoc(doc);
        exit(-1);
     }

     printf("Root: '%s', Type:%d\n",cur->name,cur->type);
     traverseAttributes(cur);

     /* go through children */
     cur = cur->children;

     while (cur != NULL) {

       traverseNode(cur,2);

       cur = cur->next;
     }

     xmlFreeDoc(doc);
     
#endif
