/* 
   COPYRIGHT 2011 KRUPCZAK.ORG, LLC.

   THIS SOFTWARE IS THE CONFIDENTIAL AND PROPRIETARY PRODUCT OF 
   KRUPCZAK.ORG, LLC.  ANY UNAUTHORIZED USE, REPRODUCTION, OR
   TRANSFER OF THIS SOFTWARE IS STRICTLY PROHIBITED.  (SUBJECT TO LIMITED
   DISTRIBUTION AND RESTRICTED DISCLOSURE ONLY.)  ALL RIGHTS RESERVED.
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <sys/types.h>

#ifndef WIN32
#include <string.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#endif

#include "pcre.h"

char *versionStr = "KorgVersion 1.0";

/* ************************************************************* */

int main(int argc, char *argv[])
{
     if (argc != 3) {
        fprintf(stderr,"Usage: %s filename regex\n\n",argv[0]);
        fprintf(stderr,"This program uses the same PCRE library, that Cartographer uses, and will allow you to test regular expressions against logfiles\n");
        exit(0);
     }


#ifndef WIN32
     struct stat statbuf;

     /* check if file exists */
     if (stat(argv[1],&statbuf) < 0) {
        fprintf(stderr,"failed to stat %s, %d\n",argv[1],errno);
        exit(-1);
     }

     if (! S_ISREG(statbuf.st_mode)) {
        fprintf(stderr,"%s is not a regular file\n",argv[1]);
        exit(-1);
     }
#endif

     scan_file(argv[1],argv[2]);
}


/* ************************************************************* */

int scan_file(char *filename, char *re)
{
     FILE *fp;
     char buffer[4096];
     int line = 0, match=0,index;
     pcre *p;
     const char *errorstr;
     int erroffset,ovector[30],ret,i;
     char substring[4096];

     if ((filename == NULL) || (re == NULL)) {
        fprintf(stderr,"scan_file: invalid args\n");
        return -1;
     }

     /* compile regular expression */
     /* options we may be interested in are PCRE_CASELESS  */
     p = pcre_compile(re,0,&errorstr,&erroffset,NULL);
     if (p == NULL) {
        fprintf(stderr,"pcre_compile failed '%s'\n",errno);
        return -1;
     }

     /* pcre_study() the pcre? */

     if ((fp = fopen(filename,"r")) == NULL) {
        perror("fopen file failed");
        return -1;
     }


     while (fgets(buffer,sizeof(buffer)-1,fp) != NULL) {
       line++;

       /* ret is number of matches */
       ret = pcre_exec(p,NULL,buffer,strlen(buffer),0,0,ovector,30);

       if (ret != -1) {
	  printf("\nLine %d matches %d '%s'\n",line,ret,buffer);
          match++;

         /* extract for captured substrings */
         /* The value returned by pcre_exec() is one more than the highest 
            numbered pair that has been set. */

         /* first pair of values in ovector identify portion of substring
            matched by pattern */

         /* subsequent pairs mark begining/end of subsequent captured 
            substrings */

         printf("Match is substring %d:%d\n",ovector[0],ovector[1]);

         index = 2;
         for (i=1; i<ret; i++) {

	     pcre_copy_substring(buffer,ovector,ret,i,substring,4095);

             printf("Substring %d is '%s'\n",i,substring);
         }

       } /* we have a match */

     } /* while fgets() */

     fclose(fp);

     printf("read %d lines, %d matches\n",line,match);

     return 0;
}

/* ************************************************************* */
