/* 
   COPYRIGHT 2004 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - win32_main.c */
/* rdk@krupczak.org */

/* 
   Service's main function and initialization code that is tied
   in with regular console/unix main(); the default win32 
   option is to run as service unless -s is given on command-line

   Either a setup program or some service-control-manager (SCM)
   configuration has to be done ahead of time prior to a service being 
   run; we can use the sc.exe utility to do this configuration 
   by hand.

*/


#ifdef WIN32

#include <winsock2.h>
#include <ws2tcpip.h>
#include <stdio.h>

#include <windows.h>
#include <pthread.h>
#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>

#include "xmp.h"
#include "socklib.h"
#include "xmp_vars.h"
#include "xmp_session.h"
#include "globals.h"

static LPTSTR ServiceName = "Cartographer";
static SERVICE_STATUS hServStatus;
static SERVICE_STATUS_HANDLE hSStat;

/* ************************************************************* */

void UpdateStatus(int NewStatus, int Check)
{
     if (Check < 0)
        hServStatus.dwCheckPoint++;
     else
        hServStatus.dwCheckPoint = Check;
     if (NewStatus >= 0)
       hServStatus.dwCurrentState = NewStatus;

     SetServiceStatus(hSStat,&hServStatus);
     return;
}

/* ************************************************************* */

void WINAPI ServiceCtrlHandler(DWORD control) 
{
     switch (control) {
       case SERVICE_CONTROL_SHUTDOWN:
       case SERVICE_CONTROL_STOP:
	  xmpd_log("Received stop/shutdown control");
 	  globalShutdown = 1;
          UpdateStatus(SERVICE_STOP_PENDING,-1);
 	  break;

       case SERVICE_CONTROL_PAUSE:
       case SERVICE_CONTROL_CONTINUE:
	  xmpd_log("Received pause/continue control");
 	  /* we dont support these */
 	  break;

       case SERVICE_CONTROL_INTERROGATE: 
 	  break;

       default:
	 if ((control > 127) && (control < 256)) {
   	    xmpd_log("Received unknown control");
 	    /* user defined */
 	    break;
         }
     }
     UpdateStatus(-1,-1); /* increment checkpoint */

     return;
}

/* ************************************************************* */
/* main start function for service; we start as a service, our
   pwd seems to be \windows\system32 perhaps because of the location
   of the service controller
*/

void WINAPI ServiceStart(DWORD argc, LPTSTR argv[])
{
     /* set server status */
     hServStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
     hServStatus.dwCurrentState = SERVICE_START_PENDING;
     hServStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | 
        SERVICE_ACCEPT_SHUTDOWN;
     hServStatus.dwWin32ExitCode = 0;
     hServStatus.dwServiceSpecificExitCode = 0;
     hServStatus.dwCheckPoint = 0;
     hServStatus.dwWaitHint = 30*1000;

     hSStat = RegisterServiceCtrlHandler(ServiceName,ServiceCtrlHandler);

     if (hSStat == 0) {
        return;
     }

     UpdateStatus(SERVICE_RUNNING,0);

     /* when init_xmpd retuns, we are done */
     init_xmpd(argc,argv); 

     UpdateStatus (SERVICE_STOPPED, 0);

     return;
}

/* ************************************************************* */

void start_win32_service(int argc, char *argv[])
{
     SERVICE_TABLE_ENTRY DispatchTable[] = {
       {ServiceName, ServiceStart },
       {NULL,NULL}
     };

     if (!StartServiceCtrlDispatcher(DispatchTable)) {
        fprintf(stderr,"Failed to start service ctrl dispatcher, %d\n",
                GetLastError());
     }

     /* if StartServiceCtrlDispatcher() succeeds, it does
        not return until service has been terminated;
        we then drop through, return to main and exit() 
     */

     return;
}

/* ************************************************************* */

#endif /* WIN32 */
