/* 
   COPYRIGHT 2004,2005 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - xmp.h */
/* rdk@krupczak.org */

/* XML message parsing definitions, internal message structures, 
   definitions, etc. */


/* ************************************************************* */
/* symbols in DLLs/binaries need to explicitly be exported so
   we define DLLExport macro to declare them for export;
   on UNIX, we do nothing so DLLExport is blank
*/

#ifdef WIN32

#define DLLExport   __declspec( dllexport )
#define DLLImport   __declspec ( dllimport )

#else

/* UNIX */
#define DLLExport
#define DLLImport  

#endif /* WIN32 */

/* ************************************************************* */

/* ************************************************************* */
/* many types, values, etc. adopted from RFC 1905 */

#ifdef SOLARIS
typedef u_longlong_t u_int64_t;
typedef unsigned int u_int32_t;
#endif

#ifdef WIN32
#define snprintf _snprintf 
#endif

/* ************************************************************* */

#ifdef WIN32

typedef _int64 u_int64_t;
typedef _int64 int64_t;
typedef unsigned int u_int32_t;
typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;
typedef unsigned char u_char;
typedef unsigned int u_int;
typedef unsigned short u_short;

#define bcopy(src,dst,n)  memcpy(dst,src,n)
#define bzero(dst,n)      memset(dst,0,n)
#define bcmp(s1,s2,n)     memcmp(s1,s2,n)

struct utsname {
     char sysname[64];    /* osname, e.g. linux, winnt, etc. */
     char nodename[64];   /* network node name */
     char release[64];    /* release level of OS */
     char version[64];    /* version of this system */
     char machine[64];    /* hardware type of machine, i686 */
};

int uname (struct utsname *);

#endif /* WIN32 */

/* ************************************************************* */
/* ************************************************************* */

/* current version of XMP */
#define XMP_VERSION   1

/* make darn sure this struct is 4-byte word-aligned!! */
typedef struct xmpWireHdr {

     /* ascii integer value indicating protocol version number;
        current version is 1 */

     char xmpVersion[4];

     /* ascii length of string no more than 12
        chars; may or may not be zero terminated
        and strings must be in decimal */

     char msgLenStr[12]; 

     /* user, authen info here, parameters, etc. ascii user/profile
        name for use with XMP messages; each message can use
        a different authenUser within a session */

     char authenUser[16];
     

} xmpWireHdr_t;

/* internal representation; values can be OR'd together to represent
   a set of valid protocol operations; operations are also sorted
   so that a comparison can be made to determine if table or scalar 
   operation */

typedef enum xmpMessageType { 
     invalidMsgType = 0,
     Response = 0x01,  /* reponses for all gets/sets/create/delete */
     GetRequest = 0x02, /* for scalars */
     SetRequest = 0x04,    /* for scalars */
     SelectTableRequest = 0x08, /* gets in tables */
     InsertTableRequest = 0x10, /* row creation */
     DeleteTableRequest = 0x20, /* row deletion */
     UpdateTableRequest = 0x40, /* row update */
     Trap =               0x80, /* trap, event, notification */
     Information =       0x100, /* a FYI */
} xmpMessageType_t;

typedef enum xmpErrorStatus { 
      invalidErrorType = -1,
      noError = 0, 
      noErrorMoreData = 1, 
      tooBig = 2, 
      noSuchName = 3, 
      badValue = 4, 
      readOnly = 5, 
      genError = 6, 
      noAccess = 7, 
      notTabular = 8, 
      wrongType = 9, 
      wrongLength = 10, 
      wrongEncoding = 11,
      wrongValue = 12, 
      noCreation = 13, 
      inconsistentValue = 14, 
      resourceUnavailable = 15,
      commitFailed = 16, 
      undoFailed = 17, 
      authorizationError = 18, 
      notWritable = 19,
      inconsistentName = 20,
      parseError = 21
} xmpErrorStatus_t;   

/* values for all integer-based types can be 64-bit */
typedef enum xmpSyntax {
    invalidSyntax = 0,
    counter = 1, 
    ipV4Address = 2, 
    ipV6Address = 3, 
    dateTime = 4, 
    gauge = 5,
    integer = 6, 
    unsignedInteger = 7, 
    bool = 8, 
    macAddress = 9, 
    physAddress = 10,
    displayString = 11, 
    binaryString = 12, 
    nullSyntax = 13, 
    table = 14, 
    floatingPoint = 15, 
    unsupportedVariable = 16,
    extendedBoolean = 17,  /* true(1), false(0), unknown(X) */
} xmpSyntax_t;

typedef struct xmpName {
    char mibName[128];
    char objName[128];
    char key[2048];    /* used for table traversals */
} xmpName_t;


/* XXX we need to retrofit xmpVariable for dynamically
   allocated buffer to hold values > 4095 bytes
   xmpPduCleanup() will need to be modified as well as
   parsing, printing, routines 
*/

typedef struct xmpVariable {
    xmpName_t   varName;
    xmpSyntax_t varSyntax;
    char        valueStr[4096];  /* all ASCII, 0-terminated */
    int         valueStrLen;     /* length of value (w/o zero-termination) */
    u_char      *valueBuf;       /* for values exceeding 4095; */
                                 /* all ASCII values, 0-terminated!!! */

    struct xmpVariable *next;

    void *entryPtr;                /* pointer to internal object */

} xmpVariable_t;

typedef struct xmpPdu {
   xmpMessageType_t msgType;
   int64_t messageID;
   xmpErrorStatus_t  errorStatus;
   u_int errorIndex;  /* 0 .. numberVariables-1 */

   /* if table PDU, these fields are valid */
   char tableMib[128];    /* mib that table is in */
   char tableName[128];   /* table MIB object name */
   char tableKey[2048];   /* key value, specific to table */
   int maxRows;           /* 0 if no max */

   u_int numberVariables;
   xmpVariable_t *varList;
   xmpVariable_t *varListTail;
  
} xmpPdu_t;

typedef struct xmpStats {
   u_int64_t inMessages;
   u_int64_t inXmlParseErrs;
   u_int64_t inErrors;
   u_int64_t inNoSuchNames;
   u_int64_t inBadValues;
   u_int64_t inTotalReqVars;
   u_int64_t inSessions;
   u_int64_t inBytes;
   u_int64_t inAuthenErrors;

   u_int64_t inGets;
   u_int64_t inSets;
   u_int64_t inResponses;
   u_int64_t inTraps;
   u_int64_t inSelects;
   u_int64_t inInserts;
   u_int64_t inDeletes;
   u_int64_t inUpdates;
   u_int64_t inInformation;

   u_int64_t outMessages;
   u_int64_t outSessions;
   u_int64_t outBytes;
   u_int64_t outErrors;
   u_int64_t outAuthenErrors;
   u_int64_t outTotalReqVars;

   u_int64_t outGets;
   u_int64_t outSets;
   u_int64_t outResponses;
   u_int64_t outTraps;
   u_int64_t outSelects;
   u_int64_t outInserts;
   u_int64_t outDeletes;
   u_int64_t outUpdates;
   u_int64_t outInformation;
  
} xmpStats_t;

/* re IANA #157665, we've been assigned port 5270 for udp/tcp */

#define XMP_PORT  5270

#define KORGOBJID "1.3.6.1.4.1.16050"

/* ************************************************************* */
/* function prototypes for using XMP functions */

int buildXmpMessage(xmpPdu_t *, u_char *, int, int *);

int parseXmpMessage(u_char *, int, xmpPdu_t *);
int parseXmpMessageByFilename(char *, xmpPdu_t *);

xmpErrorStatus_t stringToErrorStatus(char *);
int errorStatusToString(xmpErrorStatus_t , char *, int);
xmpMessageType_t stringToMessageType(char *);
int messageTypeToString(xmpMessageType_t , char *, int);
xmpSyntax_t stringToSyntax(char *);
int syntaxToString(xmpSyntax_t , char *, int );
xmpVariable_t *copyXmpVariable(xmpVariable_t *);
int copyXmpPdu(xmpPdu_t *, xmpPdu_t *);
DLLExport int xmpPduCleanup(xmpPdu_t *);
int dumpInternalXmpMessage(xmpPdu_t *);
int printXmpMessage(char *, int);

/* ************************************************************* */

